package alxXML;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.io.IOException;
import java.io.StreamTokenizer;

/**
 * Description : Classe reprsentant un lment XML avec ses attributs et ses sous-lments.
 * @author <A href=http://www.alapetite.fr.st target=_blank>Alexandre Alapetite</A>
 * @version 1.0 (06/2001)
 */
public class AlxXMLelement extends Object
{
  /**
   * Identifiant de l'lment.
   * si la balise XML est : <CODE> &lt;NODE ...&gt; </CODE>, le nom est NODE.
   */
  private String nom;
  /** Si l'lment <CODE>this</CODE> est un sous-lments, cet attribut pointe sur l'lment pre, sur null sinon. */
  protected AlxXMLelement pere=null;
  /** Table de hashage contenant des couples attributs-valeur, tous deux de type <CODE>String</CODE>. */
  private Hashtable htAttributs = new Hashtable(); //de String
  /** Contient tous les sous-lments. */
  private AlxXMLelementList vFils = new AlxXMLelementList();

  public AlxXMLelement(String nom)
  {
    super();
    this.nom=nom;
  }

  /** Ajoute (sans vrification de prsence) un nouvel attribut avec sa valeur. */
  public void addAttribut(String attribut, String valeur)
  {
    htAttributs.put(attribut,valeur);
  }

  /** @return la valeur de l'attribut de ce nom s'il existe, une chane vide sinon. */
  public String getAttribut(String attribut)
  {
    String sTemp = (String)htAttributs.get(attribut);
    return sTemp==null ? new String() : sTemp;
  }

  /**
   * Ajoute un sous-lment. (Raccourci).
   * @return vrai si l'opration s'est bien passe, faux sinon.
   * @see #getElementList
   */
  public boolean addXMLelement(AlxXMLelement alxXMLelement)
  {
    alxXMLelement.setPere(this);
    return vFils.add(alxXMLelement);
  }

  /**
   * Accs  tous les sous-lments.
   * @return l'objet grant les sous-lments
   */
  public AlxXMLelementList getElementList()
  {
    return vFils;
  }

  /** @return le nom (identifiant) de l'lment <CODE>this</CODE>. */
  public String getNom()
  {
    return nom;
  }

  /** Change le nom de l'lment <CODE>this</CODE>. */
  protected void setNom(String nom)
  {
    if (pere!=null)
    {
      boolean bTemp = pere.getElementList().remove(this);
      this.nom=nom;
      pere.addXMLelement(this);
    }
    else this.nom=nom;
  }

  /**
   * Change le pre de l'lment <CODE>this</CODE>.
   * Mthode qui est dj appele par le pre lors d'un {@link #addXMLelement addXMLelement}.
   */
  protected void setPere(AlxXMLelement alxXMLelement)
  {
    this.pere=alxXMLelement;
  }

  /**
   * Calcule en rcursif sur ses pres la profondeur de l'objet courant.
   * @return le nombre d'indentation  faire pour mettre en page l'lment courant.
   */
  protected int getIndent()
  {
    return pere==null ? 0 : pere.getIndent()+1;
  }

  /**
   * Dcodage du document XML. Lecture de la portion qui concerne l'lment courant.
   * Mthode appele dans AlxXMLdocument#decodeXMLdocument pour l'lment principal puis en rcursif pour les sous-lments.
   * @param streamTokenizer un <CODE>StreamTokenizer</CODE> ayant dj lu la &lt; ouvrante de l'lment actuel et branch sur un <CODE>BufferedReader</CODE> branch sur le fichier XML  dcoder.
   * @see AlxXMLdocument#decodeXMLdocument
   */
  protected void decodeXMLelement(StreamTokenizer streamTokenizer) throws IOException
  {
    boolean aDesFils = true;
    if (!(streamTokenizer.nextToken()==StreamTokenizer.TT_WORD))
      throw new AlxXMLioException(streamTokenizer.lineno(),"nom de l'lment attendu");
    else this.setNom(streamTokenizer.sval);
    while (streamTokenizer.nextToken()!='>')
    {
      if (streamTokenizer.ttype==StreamTokenizer.TT_EOF) throw new AlxXMLioException(streamTokenizer.lineno(),AlxXMLdocument.erreurs[1]);
      else if (streamTokenizer.ttype==StreamTokenizer.TT_WORD)
      {
        String attribut = streamTokenizer.sval;
          if (streamTokenizer.nextToken()!='=' || streamTokenizer.nextToken()!='"') //attend un [=] puis ["****"]
            throw new AlxXMLioException(streamTokenizer.lineno(),AlxXMLdocument.erreurs[2]);
          else this.addAttribut(attribut,streamTokenizer.sval);
      }
      else if (streamTokenizer.ttype=='/') aDesFils=false;
    }
    if (aDesFils)
    {
      while (streamTokenizer.nextToken()=='<' && !(streamTokenizer.nextToken()=='/'))
      {
        streamTokenizer.pushBack();
        AlxXMLelement alxElement = new AlxXMLelement(" ");
        alxElement.decodeXMLelement(streamTokenizer);
        this.addXMLelement(alxElement);
      }
      if (!(streamTokenizer.nextToken()==StreamTokenizer.TT_WORD && streamTokenizer.sval.equals(this.nom) && streamTokenizer.nextToken()=='>'))
        throw new AlxXMLioException(streamTokenizer.lineno(),AlxXMLdocument.erreurs[4]);
    }
  }

  /**
   * Mthode appele par {@link AlxXMLdocument#toString AlxXMLdocument.toString()}.
   * @return l'lment courant et ses sous-lments au format texte XML.
   */
  public String toString()
  {
    String indentation="";
    int indent = getIndent();
    for (int cpt=0; cpt<2*indent; cpt++) indentation+=" "; //pour l'indentation
    String result=indentation+"<"+nom;
    Enumeration eTemp = htAttributs.keys();
    while (eTemp.hasMoreElements())
    {
       String clef = (String)eTemp.nextElement();
       String valeur = (String)htAttributs.get(clef);
       result+=" "+clef+"=\""+valeur+"\"";
    }
    if (vFils.size()<=0) result+="/>";
    else result+=">\n"+vFils.toString()+indentation+"</"+nom+">";
    return result;
  }

}
