package grapheanim;

import java.awt.*;
import java.util.Vector;

/**
 * Description : Algorithme permettant de trouver un coloration des sommets d'un graphe.
 * L'algorithme n'assure pas que la solution est la meilleure (le plus petit nombre de couleurs)
 * @author Delphine Lara-Martil, <A href=http://www.alapetite.fr.st target=_blank>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st target=_blank>Brice Andujar</A>, Nicolas Borde
 * @version 1.0
 */
public class Coloration extends Methode_Generale
{
  private int COLORATION_MIN=1;
  private int COLORATION_MAX=Integer.MAX_VALUE;
  private int colorationMin;
  private Sommet sommet;
  private DynamicGraphData tab;
  private GrapheData Gd;
  private Color Couleurs[] = new Color[29];

  public Coloration(DynamicGraphData dgd)
  {
    super(dgd);
    tab=dgd;
    Gd=dgd.getInitGraph();
    //cration d' un tableau de couleurs
    Color r=new Color(222,69,69);
    Color ro=new Color(241,131,131);
    Color bo=new Color(177,6,6);
    Color v=new Color(139,41,201);
    Color m=new Color(218,193,234);
    Color bc=new Color(168,178,244);
    Color bp=new Color(184,211,244);
    Color cc=new Color(207,234,244);
    Color ve=new Color(28,247,229);
    Color vf=new Color(8,245,176);
    Color vfo=new Color(7,236,131);
    Color vec=new Color(40,245,84);
    Color j=new Color(184,247,14);
    Color jp=new Color(241,244,183);
    Color o=new Color(251,190,3);
    Color o2=new Color(254,150,1);
    Color o3=new Color(250,106,19);
    Color rou=new Color(253,12,0);
    Color vio=new Color(129,9,129);
    Color ma=new Color(148,103,54);
    Color oc=new Color(246,212,176);
    Couleurs[0]=Color.cyan;
    Couleurs[1]=r;
    Couleurs[2]=Color.orange;
    Couleurs[3]=v;
    Couleurs[4]=Color.pink;
    Couleurs[5]=bc;
    Couleurs[6]=Color.yellow;
    Couleurs[7]=ve;
    Couleurs[8]=beige;
    Couleurs[9]=bo;
    Couleurs[10]=Color.green;
    Couleurs[11]=ro;
    Couleurs[12]=Color.magenta;
    Couleurs[13]=bp;
    Couleurs[14]=Color.red;
    Couleurs[15]=vf;
    Couleurs[16]=o;
    Couleurs[17]=vio;
    Couleurs[18]=cc;
    Couleurs[19]=o2;
    Couleurs[20]=m;
    Couleurs[21]=ma;
    Couleurs[22]=jp;
    Couleurs[23]=rou;
    Couleurs[24]=vf;
    Couleurs[25]=oc;
    Couleurs[26]=j;
    Couleurs[27]=vec;
    Couleurs[28]=o3;
  }

  public void Explorer()
  {
    int nbrsommets=Gd.nbrSommet();

    String s=tab.getOrigine();
    String nom="";
    Sommet a=NOM(s);
    Sommet OldOld=NOM(s).clonePoints();
    int coloration=1;
    //rcupration des successeurs de l'origine
    Vector successeurs=a.getAllSuccesseurs();
    DynamicData DD2 = new DynamicData();
    String algo="\nParcours de tous les successeurs de: "+s;
    String commentaire="\nSommet visit: "+s;
    DynamicData DD3;// =new DynamicData();
    //on regarde la couleurs de tous ses successeurs et on attribue
    //la plus petite couleur possible au sommet
    for(int i=0;i<a.nbrSuccesseur();i++)
    {
      nom= ((Successeur)(successeurs.elementAt(i))).getNom();
      algo+="\nSuccesseur: "+nom;
      commentaire += "\nCouleur du successeur "+nom+" : "+((Successeur)(successeurs.elementAt(i))).getColoration();
      //on colore les artes en blanc pour suivre l'xcution de l'algo
      SauverCouleurArrete(DD2,Color.white,s,nom);
      SauverCouleurArrete(DD2,Color.white,nom,s);

      if(((Successeur)(successeurs.elementAt(i))).getColoration()!=0)
        while(coloration==((Successeur)(successeurs.elementAt(i))).getColoration())
          coloration++;
    }
    commentaire+="\n\n==> Couleur du sommet: "+coloration+"\n";
    a.setColoration(coloration);
    DD2.setAlgo(algo);
    DD2.setCommentaire(commentaire);
    get_DGD().addStep(DD2);
    //coloration du sommet avec sa bonne couleur
    SauverCouleurSommet(Couleurs[coloration], s);
    DD2=new DynamicData();
    for(int i=0;i<a.nbrSuccesseur();i++)
    {
      //on remet les artes en noir pour tester un nouveau sommet
     SauverCouleurArrete(DD2,Color.black,s,((Successeur)(successeurs.elementAt(i))).getNom());
     SauverCouleurArrete(DD2,Color.black,((Successeur)(successeurs.elementAt(i))).getNom(),s);
    }
    get_DGD().addStep(DD2);
    DD2 = new DynamicData();
    a.setColoration(coloration);
    coloration=1;
    Vector success=new Vector();
    //on effectue la meme chose pour tous les sommets autres que l'origine
    for(int i=0;i<nbrsommets;i++)
    {
      DD2=new DynamicData();
      DD3=new DynamicData();
      if(((Sommet)(Gd.sommet.elementAt(i))).getNom()!=s)
      {
        String n=((Sommet)(Gd.sommet.elementAt(i))).getNom();
        algo="\nParcours de tous les successeurs de: "+n;
        commentaire="\nSommet visit: "+n;
        SauverCouleurSommet(Color.blue,n);
        Sommet l=NOM(n);
        success=l.getAllSuccesseurs();
        DD2=new DynamicData();
        for(int j=0;j<l.nbrSuccesseur();j++)
        {
          nom=((Successeur)(success.elementAt(j))).getNom();
          algo+="\nSuccesseur: "+nom;
          Sommet t=NOM(nom);
          commentaire+="\nCouleur du successeur "+nom+" : "+t.getColoration();

          SauverCouleurArrete(DD2,Color.white,n,nom);
          SauverCouleurArrete(DD2,Color.white,nom,n);
          if(t.getColoration()!=0)
            while(coloration==t.getColoration())
              coloration++;
        }

        int comp=1;
        while(comp!=0)
        {
          comp=0;
          for(int j=0;j<l.nbrSuccesseur();j++)
          {
            nom=((Successeur)(success.elementAt(j))).getNom();
            Sommet t=NOM(nom);
            if(coloration==t.getColoration())
            {
              coloration++;
              comp++;
            }
          }
        }
        commentaire+="\n\n==>Couleur du sommet: "+coloration+"\n";
        DD2.setCommentaire(commentaire);
        DD2.setAlgo(algo);

        get_DGD().addStep(DD2);

        DD2=new DynamicData();
        l.setColoration(coloration);
        SauverCouleurSommet(Couleurs[coloration], n);
        for(int k=0;k<l.nbrSuccesseur();k++)
          {SauverCouleurArrete(DD2,Color.black,n,((Successeur)(success.elementAt(k))).getNom());
            SauverCouleurArrete(DD2,Color.black,((Successeur)(success.elementAt(k))).getNom(),n);}
        get_DGD().addStep(DD2);
      }
      coloration=1;
    }
  }

  public void Initialiser()
  {
    for (int i = 0; i < Taille(); i++)
    {
      Sommet P = GD().sommetAt(i);
      for (int j=0;j < P.nbrSuccesseur(); j++)
        P.getSuccesseurAt(j).setCouleur(Color.black);
      P.setCouleur(Color.gray);
      P.set_Dist(100000);
      P.set_fermes(false);
    }
    DynamicData DD =new DynamicData();
    DD.setAlgo("\nOrigine: " + DGD.getOrigine());
    DD.setCommentaire("\nINITIALISATION de l'algorithme de coloration\n " );

    Sommet Old = NOM(DGD.getOrigine()).clonePoints();
    NOM(DGD.getOrigine()).setCouleur(Color.blue);
    Sommet New = NOM(DGD.getOrigine()).clonePoints();
    DD.addSommet(Old, New);

    get_DGD().addStep(DD);

    NOM(DGD.getOrigine()).set_Parent("NIL");    //on met le parent  'NIL'
  }

}
