package grapheanim;

import java.awt.*;
import java.io.*;
import java.util.Vector;

/** Classe contenant un graphe de dpart ainsi que toutes les tapes d'un algorithme. */
public class DynamicGraphData extends Object implements Serializable
{
  private GrapheData InitGraph, VisualGD;
  private Vector Steps;
  private int StepSize;
  private int CurrentStep;
  private int FinalStep;
  private String origine, arrivee, algo;

  //les constructeurs
  DynamicGraphData()
  {
    InitGraph = new GrapheData();
    VisualGD = new GrapheData();
    Steps = new Vector();
    StepSize = CurrentStep = FinalStep = 0;
    origine = arrivee = algo = "";
  }

  DynamicGraphData(DynamicGraphData dgd)
  {
    Copy(dgd);
  }

  public void Copy(DynamicGraphData dgd)
  {
    InitGraph = new GrapheData(dgd.InitGraph);
    VisualGD = new GrapheData(InitGraph);
    Steps = dgd.Steps;
    StepSize = dgd.StepSize;
    CurrentStep = dgd.CurrentStep;
    FinalStep = dgd.FinalStep;
    origine = dgd.origine;
    arrivee = dgd.arrivee;
    algo = dgd.algo;
  }

  public void Init(GrapheData Data, int Size, int Final)
  {
    InitGraph = Data;
    StepSize = Size;
    FinalStep = Final;
    CurrentStep = 0;
  }

  // les accesseurs
  public void setInitGraph(GrapheData g) { InitGraph = g; VisualGD = new GrapheData(g); }
  public GrapheData getInitGraph() { return InitGraph; }
  public void setSteps(Vector v) { Steps = v; }
  public Vector getSteps() { return Steps; }
  public void setStepSize(int i) { StepSize = i; }
  public int getStepSize() { return StepSize;  }
  public void setCurrentStepIndex(int i) { gotoStep(i); }
  public int getCurrentStepIndex() { return CurrentStep;  }
  public void setFinalStep(int i) { FinalStep = i; }
  public int getFinalStep() { return FinalStep;  }
  public int nbrSteps() { return Steps.size(); }
  public void setOrigine(String s) { origine = s; }
  public String getOrigine() { return origine; }
  public void setArrivee(String s) { arrivee = s; }
  public String getArrivee() { return arrivee; }
  public void setAlgo(String s) { algo = s; }
  public String getAlgo() { return algo; }
  public DynamicData getStepAt(int i) { return (DynamicData) Steps.elementAt(i); }

  public void gotoFirstStep()
  {
    VisualGD = new GrapheData(InitGraph);
    CurrentStep = 0;
  }

  public void addStep(DynamicData Data)
  {
    Steps.addElement(Data);
  }

  public int getCurrentNumStep()
  {
    int Tmp = CurrentStep*StepSize;
    if (Tmp>FinalStep) Tmp = FinalStep;
    return Tmp;
  }

  public DynamicData getCurrentStep()
  {
    return (DynamicData)Steps.elementAt(CurrentStep);
  }

  public boolean gotoNextStep()
  {
    boolean Result = false;
    if(CurrentStep < Steps.size())
    {
      Vector ss = getStepAt(CurrentStep).getNewSommets();
      Sommet sommet;
      for(int i=0;i<ss.size(); i++)
      {
        sommet = (Sommet)ss.elementAt(i);
        VisualGD.setSommetAt(sommet,VisualGD.indexOf(sommet.getNom()));
      }
      CurrentStep++;
      Result = true;
    }
    return Result;
  }

  public boolean gotoPreviousStep()
  {
    boolean Result = false;
    if(CurrentStep > 0)
    {
      CurrentStep--;
      Vector ss = getStepAt(CurrentStep).getOldSommets();
      Sommet sommet;
      for(int i=0;i<ss.size(); i++)
      {
        sommet = (Sommet)ss.elementAt(i);
        VisualGD.setSommetAt(sommet,VisualGD.indexOf(sommet.getNom()));
      }
      Result = true;
    }
    return Result;
  }

  public void gotoStep(int num)
  {
    if (num > CurrentStep)
      while(CurrentStep != num) gotoNextStep();
    else
      while(CurrentStep != num) gotoPreviousStep();
  }

  public void sauver(Frame fr)
  {
    FileDialog sauve = new FileDialog(fr, "Sauver le graphe", FileDialog.SAVE);
    sauve.setFile("graphe"+Long.toString(System.currentTimeMillis()/1000)+".grf");
    sauve.show();
    String nomFichier = sauve.getDirectory()+sauve.getFile();
    if (sauve.getFile() != null)
    {
      FileXML f = new FileXML();
      boolean b = f.SaveToFile(nomFichier,this);
      /** @todo a faire dans une boite de dialogue */
      if (!b) System.out.println(f.getLastError());
    }
  }

  public void ouvrir(Frame fr)
  {
    FileDialog ouvrir = new FileDialog(fr, "Ouvrir un graphe", FileDialog.LOAD);
    ouvrir.setFile("*.grf");
    ouvrir.show();
    String nomFichier = ouvrir.getDirectory()+ouvrir.getFile();
    if (ouvrir.getFile() != null)
    {
      FileXML f = new FileXML();
      DynamicGraphData d = f.ReadFromFile(nomFichier);
      if (d != null) Copy(d);
      /** @todo a faire dans une boite de dialogue */
      else System.out.println(f.getLastError());
    }
  }

  public void afficher(Graphics g, boolean arriv, boolean editeur)
  {
    InitGraph.afficher(g, getAlgo());
    if (!editeur) VisualGD.afficher(g, getAlgo());
    //affichage de l'origine & arrivee
    if (origine.length()>0)
    {
      g.setColor(Color.green);
      Sommet sommet = InitGraph.getSommet(origine);
      g.drawOval(sommet.getX(),sommet.getY(), 25, 25);
    }
    if (arriv && (arrivee.length()>0))
    {
      g.setColor(Color.red);
      Sommet sommet = InitGraph.getSommet(arrivee);
      g.drawOval(sommet.getX(),sommet.getY(),25,25);
    }
  }

  /**
   * @deprecated
   * @see #afficher(Graphics,boolean,boolean)
   */
  public void afficher(Graphics g)
  {
    InitGraph.afficher(g, getAlgo());
    if (!getAlgo().equals("editeur")) VisualGD.afficher(g, getAlgo());
    //affichage de l'origine & arrivee
    if (origine.length()>0)
    {
      //if (CurrentStep == 0)
      //{
      //  g.setColor(new Color(238,208,161));
      //  g.fillOval(InitGraph.getSommet(origine).getX(),
      //  InitGraph.getSommet(origine).getY(), 25, 25);
      //}
      g.setColor(Color.green);
      Sommet sommet = InitGraph.getSommet(origine);
      g.drawOval(sommet.getX(),sommet.getY(), 25, 25);
    }
    if ((arrivee.length()>0) &&
       ((getAlgo().equals("Flot")) ||
       (getAlgo().equals("A*")) ||
       (getAlgo().equals("A")) ||
       (getAlgo().equals("Lexico"))))
    {
      g.setColor(Color.red);
      Sommet sommet = InitGraph.getSommet(arrivee);
      g.drawOval(sommet.getX(),sommet.getY(),25,25);
    }
  }

}
