package grapheanim;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.util.Properties;

/**
 * Classe permettant de grer un sommet du graphe.
 * @version 2.0 Ajout d'une table de hachage pour viter les parcours de successeurs.
 */
public class Sommet implements Serializable
{
  private Point coordonnee;
  private String nom;
  private Color couleur;
  private int poid;
  private int flot;
  private Vector successeur;
  /**
   * Table de hachage contenant aussi les successeurs. Mmes pointeurs que dans le <CODE>Vector</CODE> successeur.
   * @see #successeur
   */
  private Hashtable htSuccesseurs;
  protected int heuristique;    //pour A* ou A_vol
  protected String Dist_Lexico; //pour largeur lexicographique
  protected int couche;
  private int coloration;
  private boolean couple;

  static final int DIST_MAX = 100000;

  protected String Parent;
  //protected int nb_Voisin;  nbrSuccesseur()
  protected boolean ouverts;
  protected boolean fermes;

  //les constructeurs
  public Sommet(Point p, String s)
  {
    coordonnee = new Point(p);
    nom = s;
    couleur = Color.gray;
    poid = DIST_MAX;
    flot = 0;
    successeur = new Vector();
    htSuccesseurs = new Hashtable();
    Parent = s;      //Parent(y) <- y
    ouverts = false;
    fermes = false;
    getLocation().setLocation(p);  //on met les coordonnees du point dans 'coordonnee'
    heuristique = 0;
    Dist_Lexico = "";
    couche = 0;
    coloration = 0;
    couple=false;
  }

  public Sommet()
  {
    coordonnee = new Point();
    successeur = new Vector();
    htSuccesseurs = new Hashtable();
    Dist_Lexico = "";
  }

  //les accesseurs
  public boolean getCouple(){return couple;}
  public void setCouple(boolean actif) { this.couple=actif; }
  public Point getLocation() { return coordonnee; }
  public int getX() { return coordonnee.x; }
  public int getY() { return coordonnee.y; }
  public void setX(int i) { coordonnee.x = i; }
  public void setY(int i) { coordonnee.y = i; }
  public void setLocation(Point p) { coordonnee.setLocation(p); }
  public String getNom() { return nom; }
  public void setNom(String s) { nom = s; }
  public Color getCouleur() { return couleur; }
  public void setCouleur(Color c) { couleur = c; }
  public int getPoid() { return poid; }
  public void setPoid(int i) { poid = i; }
  public int getFlot() { return flot; }
  public void setFlot(int i) { flot = i; }
  public int getHeuristique() { return heuristique; }
  public void setHeuristique(int i) { heuristique = i; }
  public String getDistLexico() { return Dist_Lexico; }
  public void setDistLexico(String s) { Dist_Lexico = s; }
  public int nbrSuccesseur() { return successeur.size(); }
  public void addSuccesseur(String s, int i){ addSuccesseur(new Successeur(s, i)); }
  public void addSuccesseur(String s, int i, int f) { addSuccesseur(new Successeur(s, i, f)); }
  public void addSuccesseur(Successeur s)
  {
    /*int position = indexOfSuccesseur(s.getNom());
    if (position>0) setSuccesseurAt(position,s); //un successeur existe dj
    else*/
    successeur.addElement(s);
    htSuccesseurs.put(s.getNom(),s);
  }
  public void removeSuccesseur(String s)
  {
    /*for (int i=successeur.size()-1; i>=0 ; i--)
    {
      Successeur suc = (Successeur)successeur.elementAt(i);
      if (suc.getNom().equals(s)) successeur.removeElementAt(i);
    }*/
    Successeur aSuccesseur = (Successeur)htSuccesseurs.remove(s);
    successeur.remove(aSuccesseur);
  }
  /*public void chercheTousPredecesseurs(GrapheData gd)
  {
    predecesseurs = new Vector();
    htPredecesseurs = new Hashtable();
    Vector success;
    boolean trouve;
    for (int cpt=0; cpt<gd.nbrSommet(); cpt++)
    {
      success = gd.sommetAt(cpt).getAllSuccesseurs();
      trouve=false;
      int cpt2=0;
      while (!trouve && (cpt2<success.size()))
      {
        if (((Successeur)success.elementAt(cpt2)).getNom().equals(nom))
        {
          String nomSucc = gd.sommetAt(cpt).get_Nom();
          predecesseurs.add(nomSucc);
          htPredecesseurs.put(nomSucc,gd.sommetAt(cpt));
          trouve=true;
        }
        cpt2++;
      }
    }
  }*/
  public Vector getAllSuccesseurs() { return successeur; }
  public void removeAllSuccesseurs()
  {
    successeur.clear(); //removeAllElements();
    htSuccesseurs.clear();
  }
  public int indexOfSuccesseur(String s)
  {
    /*boolean trouve=false;
    int cpt=0;
    while(!trouve && (cpt<successeur.size()))
    {
      trouve = ((Successeur)successeur.elementAt(cpt)).getNom().equals(s);
      cpt++;
    }
    return trouve ? --cpt : -1;*/
    Successeur aSuccesseur = (Successeur)htSuccesseurs.get(s);
    return successeur.indexOf(aSuccesseur);
  }
  public void setSuccesseurAt(Successeur succ, int i)
  {
    htSuccesseurs.remove(((Successeur)successeur.elementAt(i)).getNom());
    htSuccesseurs.put(succ.getNom(),succ);
    successeur.setElementAt(succ,i);
  }
  public Successeur getSuccesseurAt(int i) { return (Successeur) successeur.elementAt(i); }
  public Successeur getSuccesseur(String s)
  {
    /*for (int i = 0; i < successeur.size(); i++)
    {
      Successeur suc = (Successeur)successeur.elementAt(i);
      if (suc.getNom().equals(s)) return suc;
    }
    return null;*/
    return (Successeur)htSuccesseurs.get(s);
  }
  public void renameSuccesseur(String s1, String s2)
  {
    Successeur aSuccesseur = (Successeur)htSuccesseurs.remove(s1);
    aSuccesseur.setNom(s2);
    htSuccesseurs.put(aSuccesseur.getNom(),aSuccesseur);
    /*for (int i = 0; i < successeur.size(); i++)
    {
      Successeur suc = (Successeur)successeur.elementAt(i);
      if (suc.getNom().equals(s1)) suc.setNom(s2);
    }*/
  }

  public String get_Nom() {return getNom();}
  public String get_Parent() {return Parent;}
  public boolean get_ouverts() {return ouverts;}
  public boolean get_fermes() {return fermes;}
  public int get_Dist() {return getPoid();}
  public int get_Flot() {return getFlot();}
  public int getCouche() {return couche;}
  public int getColoration() {return coloration;}
  public void setColoration(int coloration)
  {
    this.coloration=coloration;
    setPoid(coloration);
  }

  //accesseurs pour l'algo A*
  public int get_h() {return heuristique;}
  public int get_X() {return getLocation().x;}
  public int get_Y() {return getLocation().y;}

  //accesseurs li  l'algo lexicographique
  public String get_Dist_Lexico() {return Dist_Lexico;}

  //accesseur du tableau des sucesseurs
  public Successeur GET_S() {return (Successeur)getAllSuccesseurs().firstElement();}
  public Successeur GET_S(int place) {return (Successeur)getAllSuccesseurs().elementAt(place);}
  public String get_Voisin_S(int place) {return GET_S(place).getNom();}
  public int get_Valeur_S(int place) {return GET_S(place).getPoid();}
  public int get_ValeurFlot_S(int place) {return GET_S(place).getFlot();}

  //Affectations
  public void set_Parent(String a) {Parent = a;}
  public void set_ouverts(boolean a) {ouverts = a;}
  public void set_fermes(boolean a) {fermes = a;}
  public void set_Dist(int d) {setPoid(d);}
  public void set_Flot(int f) {setFlot(f);}
  public void setCouche(int d) {couche = d;}

  //affectation pour l'algo A*
  public void set_h(int i) {heuristique = i;}

  //affectation li  l'algo lexicographique
  public void set_Dist_Lexico(String S) {Dist_Lexico = S;}
  public void set_Successeur(Vector V) {successeur = V;}

  public Sommet clonePoints()
  {
    Sommet P = new Sommet(new Point(get_X(),get_Y()), get_Nom());
    P.setLocation(getLocation());
    P.setNom(getNom());
    P.setCouleur(getCouleur());
    P.set_Dist_Lexico(get_Dist_Lexico());
    P.set_Parent(Parent);
    P.set_ouverts(ouverts);
    P.set_fermes(fermes);
    P.set_Dist(getPoid());
    P.set_Flot(getFlot());
    P.setHeuristique(getHeuristique());
    //P.setCouche(getCouche());
    P.setColoration(getColoration());
    //P.setPoid(getPoid());
    for(int i=0;i<successeur.size(); i++)
      P.addSuccesseur(new Successeur (getSuccesseurAt(i)));
    return P;
  }

  public String toString()
  {
    return this.getNom();
  }
}
